#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""User supplied CAN driver.

Run as:

.. code-block:: shell

    python xcp_user_supplied_driver.py -c conf_can_user.toml
"""
from pyxcp.cmdline import ArgumentParser

__copyright__ = """
    pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2009-2020 by Christoph Schueler <cpu12.gems@googlemail.com>

   All Rights Reserved

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""


from pyxcp.transport.can import CanInterfaceBase


class MyCI(CanInterfaceBase):
    """

    Relevant options in your configuration file (e.g. conf_can_user.toml):

    TRANSPORT = "CAN"
    CAN_DRIVER = "MyCI"     # The name of your custom driver class.

    """

    def init(self, parent, receive_callback):
        self.parent = parent

    def connect(self):
        pass

    def close(self):
        pass

    def getTimestampResolution(self):
        pass

    def read(self):
        pass

    def transmit(self, payload: bytes):
        print("\tTX-PAYLOAD", payload)


ap = ArgumentParser(description="User supplied CAN driver.")
with ap.run() as x:
    x.connect()
    x.disconnect()

"""
This do-nothing example will output

    TX-PAYLOAD b'\xff\x00'

and then timeout (0xff is the service code for CONNECT_REQ).
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Checksum calculation for memory ranges

.. [1] XCP Specification, BUILD_CHECKSUM service.
"""

__copyright__ = """
    pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2009-2019 by Christoph Schueler <cpu12.gems@googlemail.com>

   All Rights Reserved

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import enum
import struct
import zlib


class Algorithm(enum.IntEnum):
    """Enumerates available checksum algorithms"""

    XCP_ADD_11 = 1
    XCP_ADD_12 = 2
    XCP_ADD_14 = 3
    XCP_ADD_22 = 4
    XCP_ADD_24 = 5
    XCP_ADD_44 = 6
    XCP_CRC_16 = 7
    XCP_CRC_16_CITT = 8
    XCP_CRC_32 = 9
    XCP_USER_DEFINED = 10


CRC16 = (
    0x0000,
    0x8005,
    0x800F,
    0x000A,
    0x801B,
    0x001E,
    0x0014,
    0x8011,
    0x8033,
    0x0036,
    0x003C,
    0x8039,
    0x0028,
    0x802D,
    0x8027,
    0x0022,
    0x8063,
    0x0066,
    0x006C,
    0x8069,
    0x0078,
    0x807D,
    0x8077,
    0x0072,
    0x0050,
    0x8055,
    0x805F,
    0x005A,
    0x804B,
    0x004E,
    0x0044,
    0x8041,
    0x80C3,
    0x00C6,
    0x00CC,
    0x80C9,
    0x00D8,
    0x80DD,
    0x80D7,
    0x00D2,
    0x00F0,
    0x80F5,
    0x80FF,
    0x00FA,
    0x80EB,
    0x00EE,
    0x00E4,
    0x80E1,
    0x00A0,
    0x80A5,
    0x80AF,
    0x00AA,
    0x80BB,
    0x00BE,
    0x00B4,
    0x80B1,
    0x8093,
    0x0096,
    0x009C,
    0x8099,
    0x0088,
    0x808D,
    0x8087,
    0x0082,
    0x8183,
    0x0186,
    0x018C,
    0x8189,
    0x0198,
    0x819D,
    0x8197,
    0x0192,
    0x01B0,
    0x81B5,
    0x81BF,
    0x01BA,
    0x81AB,
    0x01AE,
    0x01A4,
    0x81A1,
    0x01E0,
    0x81E5,
    0x81EF,
    0x01EA,
    0x81FB,
    0x01FE,
    0x01F4,
    0x81F1,
    0x81D3,
    0x01D6,
    0x01DC,
    0x81D9,
    0x01C8,
    0x81CD,
    0x81C7,
    0x01C2,
    0x0140,
    0x8145,
    0x814F,
    0x014A,
    0x815B,
    0x015E,
    0x0154,
    0x8151,
    0x8173,
    0x0176,
    0x017C,
    0x8179,
    0x0168,
    0x816D,
    0x8167,
    0x0162,
    0x8123,
    0x0126,
    0x012C,
    0x8129,
    0x0138,
    0x813D,
    0x8137,
    0x0132,
    0x0110,
    0x8115,
    0x811F,
    0x011A,
    0x810B,
    0x010E,
    0x0104,
    0x8101,
    0x8303,
    0x0306,
    0x030C,
    0x8309,
    0x0318,
    0x831D,
    0x8317,
    0x0312,
    0x0330,
    0x8335,
    0x833F,
    0x033A,
    0x832B,
    0x032E,
    0x0324,
    0x8321,
    0x0360,
    0x8365,
    0x836F,
    0x036A,
    0x837B,
    0x037E,
    0x0374,
    0x8371,
    0x8353,
    0x0356,
    0x035C,
    0x8359,
    0x0348,
    0x834D,
    0x8347,
    0x0342,
    0x03C0,
    0x83C5,
    0x83CF,
    0x03CA,
    0x83DB,
    0x03DE,
    0x03D4,
    0x83D1,
    0x83F3,
    0x03F6,
    0x03FC,
    0x83F9,
    0x03E8,
    0x83ED,
    0x83E7,
    0x03E2,
    0x83A3,
    0x03A6,
    0x03AC,
    0x83A9,
    0x03B8,
    0x83BD,
    0x83B7,
    0x03B2,
    0x0390,
    0x8395,
    0x839F,
    0x039A,
    0x838B,
    0x038E,
    0x0384,
    0x8381,
    0x0280,
    0x8285,
    0x828F,
    0x028A,
    0x829B,
    0x029E,
    0x0294,
    0x8291,
    0x82B3,
    0x02B6,
    0x02BC,
    0x82B9,
    0x02A8,
    0x82AD,
    0x82A7,
    0x02A2,
    0x82E3,
    0x02E6,
    0x02EC,
    0x82E9,
    0x02F8,
    0x82FD,
    0x82F7,
    0x02F2,
    0x02D0,
    0x82D5,
    0x82DF,
    0x02DA,
    0x82CB,
    0x02CE,
    0x02C4,
    0x82C1,
    0x8243,
    0x0246,
    0x024C,
    0x8249,
    0x0258,
    0x825D,
    0x8257,
    0x0252,
    0x0270,
    0x8275,
    0x827F,
    0x027A,
    0x826B,
    0x026E,
    0x0264,
    0x8261,
    0x0220,
    0x8225,
    0x822F,
    0x022A,
    0x823B,
    0x023E,
    0x0234,
    0x8231,
    0x8213,
    0x0216,
    0x021C,
    0x8219,
    0x0208,
    0x820D,
    0x8207,
    0x0202,
)

CRC16_CCITT = (
    0x0000,
    0x1021,
    0x2042,
    0x3063,
    0x4084,
    0x50A5,
    0x60C6,
    0x70E7,
    0x8108,
    0x9129,
    0xA14A,
    0xB16B,
    0xC18C,
    0xD1AD,
    0xE1CE,
    0xF1EF,
    0x1231,
    0x0210,
    0x3273,
    0x2252,
    0x52B5,
    0x4294,
    0x72F7,
    0x62D6,
    0x9339,
    0x8318,
    0xB37B,
    0xA35A,
    0xD3BD,
    0xC39C,
    0xF3FF,
    0xE3DE,
    0x2462,
    0x3443,
    0x0420,
    0x1401,
    0x64E6,
    0x74C7,
    0x44A4,
    0x5485,
    0xA56A,
    0xB54B,
    0x8528,
    0x9509,
    0xE5EE,
    0xF5CF,
    0xC5AC,
    0xD58D,
    0x3653,
    0x2672,
    0x1611,
    0x0630,
    0x76D7,
    0x66F6,
    0x5695,
    0x46B4,
    0xB75B,
    0xA77A,
    0x9719,
    0x8738,
    0xF7DF,
    0xE7FE,
    0xD79D,
    0xC7BC,
    0x48C4,
    0x58E5,
    0x6886,
    0x78A7,
    0x0840,
    0x1861,
    0x2802,
    0x3823,
    0xC9CC,
    0xD9ED,
    0xE98E,
    0xF9AF,
    0x8948,
    0x9969,
    0xA90A,
    0xB92B,
    0x5AF5,
    0x4AD4,
    0x7AB7,
    0x6A96,
    0x1A71,
    0x0A50,
    0x3A33,
    0x2A12,
    0xDBFD,
    0xCBDC,
    0xFBBF,
    0xEB9E,
    0x9B79,
    0x8B58,
    0xBB3B,
    0xAB1A,
    0x6CA6,
    0x7C87,
    0x4CE4,
    0x5CC5,
    0x2C22,
    0x3C03,
    0x0C60,
    0x1C41,
    0xEDAE,
    0xFD8F,
    0xCDEC,
    0xDDCD,
    0xAD2A,
    0xBD0B,
    0x8D68,
    0x9D49,
    0x7E97,
    0x6EB6,
    0x5ED5,
    0x4EF4,
    0x3E13,
    0x2E32,
    0x1E51,
    0x0E70,
    0xFF9F,
    0xEFBE,
    0xDFDD,
    0xCFFC,
    0xBF1B,
    0xAF3A,
    0x9F59,
    0x8F78,
    0x9188,
    0x81A9,
    0xB1CA,
    0xA1EB,
    0xD10C,
    0xC12D,
    0xF14E,
    0xE16F,
    0x1080,
    0x00A1,
    0x30C2,
    0x20E3,
    0x5004,
    0x4025,
    0x7046,
    0x6067,
    0x83B9,
    0x9398,
    0xA3FB,
    0xB3DA,
    0xC33D,
    0xD31C,
    0xE37F,
    0xF35E,
    0x02B1,
    0x1290,
    0x22F3,
    0x32D2,
    0x4235,
    0x5214,
    0x6277,
    0x7256,
    0xB5EA,
    0xA5CB,
    0x95A8,
    0x8589,
    0xF56E,
    0xE54F,
    0xD52C,
    0xC50D,
    0x34E2,
    0x24C3,
    0x14A0,
    0x0481,
    0x7466,
    0x6447,
    0x5424,
    0x4405,
    0xA7DB,
    0xB7FA,
    0x8799,
    0x97B8,
    0xE75F,
    0xF77E,
    0xC71D,
    0xD73C,
    0x26D3,
    0x36F2,
    0x0691,
    0x16B0,
    0x6657,
    0x7676,
    0x4615,
    0x5634,
    0xD94C,
    0xC96D,
    0xF90E,
    0xE92F,
    0x99C8,
    0x89E9,
    0xB98A,
    0xA9AB,
    0x5844,
    0x4865,
    0x7806,
    0x6827,
    0x18C0,
    0x08E1,
    0x3882,
    0x28A3,
    0xCB7D,
    0xDB5C,
    0xEB3F,
    0xFB1E,
    0x8BF9,
    0x9BD8,
    0xABBB,
    0xBB9A,
    0x4A75,
    0x5A54,
    0x6A37,
    0x7A16,
    0x0AF1,
    0x1AD0,
    0x2AB3,
    0x3A92,
    0xFD2E,
    0xED0F,
    0xDD6C,
    0xCD4D,
    0xBDAA,
    0xAD8B,
    0x9DE8,
    0x8DC9,
    0x7C26,
    0x6C07,
    0x5C64,
    0x4C45,
    0x3CA2,
    0x2C83,
    0x1CE0,
    0x0CC1,
    0xEF1F,
    0xFF3E,
    0xCF5D,
    0xDF7C,
    0xAF9B,
    0xBFBA,
    0x8FD9,
    0x9FF8,
    0x6E17,
    0x7E36,
    0x4E55,
    0x5E74,
    0x2E93,
    0x3EB2,
    0x0ED1,
    0x1EF0,
)


def reflect(data, nBits):
    """Reflect data, i.e. reverse bit order.

    Parameters
    ----------
    data : int
    nBits : int
        width in bits of `data`
    """
    reflection = 0x00000000
    for bit in range(nBits):
        if data & 0x01:
            reflection |= 1 << ((nBits - 1) - bit)
        data = data >> 1
    return reflection


class Crc16:
    """Calculate CRC (16-bit)


    Parameters
    ----------
    table: list-like
        lookup table for CRC calculation
    initalRemainder : int
        value to start with
    finalXorValue : int
        final XOR value
    reflectData : bool
        reflect input data
    reflectRemainder : bool
        reflect output data

    .. [1] A PAINLESS GUIDE TO CRC ERROR DETECTION ALGORITHMS
           http://www.ross.net/crc/download/crc_v3.txt
    .. [2] Understanding and implementing CRC (Cyclic Redundancy Check)
           calculation
           http://www.sunshine2k.de/articles/coding/crc/understanding_crc.html
    .. [3] Online CRC calculator
           http://zorc.breitbandkatze.de/crc.html
    """

    WIDTH = 16

    def __init__(
        self, table, initalRemainder, finalXorValue, reflectData, reflectRemainder
    ):
        self.table = table
        self.initalRemainder = initalRemainder
        self.finalXorValue = finalXorValue
        self.reflectData = reflectData
        self.reflectRemainder = reflectRemainder

    def __call__(self, frame):
        remainder = self.initalRemainder
        for ch in frame:
            data = self.reflectIn(ch, remainder)
            remainder = (self.table[data] ^ (remainder << 8)) & 0xFFFF
        return self.reflectOut(remainder)

    def reflectIn(self, ch, remainder):
        if self.reflectData:
            return (reflect(ch, 8) ^ (remainder >> (self.WIDTH - 8))) & 0xFF
        else:
            return (ch ^ (remainder >> (self.WIDTH - 8))) & 0xFF

    def reflectOut(self, remainder):
        if self.reflectRemainder:
            return reflect(remainder, 16) ^ self.finalXorValue
        else:
            return remainder ^ self.finalXorValue


def adder(modulus):
    """Factory function for modulus adders

    Parameters
    ----------
    modulus : int
        modulus to use

    Returns
    -------
    function
        adder function

    Examples
    --------
    >>> a256=adder(256)
    >>> a256([11, 22, 33, 44, 55, 66, 77, 88, 99])
    239

    """

    def add(frame):
        return sum(frame) % modulus

    return add


def wordSum(modulus, step):
    """Factory function for (double-)word modulus sums

    Parameters
    ----------
    modulus : int
    step : [2, 4]
        2 - word wise
        4 - double-word wise

    Returns
    -------
    function
        summation function
    """

    def add(frame):
        if step == 2:
            mask = "<H"
        elif step == 4:
            mask = "<I"
        else:
            raise NotImplementedError("Only WORDs or DWORDs are supported.")
        x = [
            struct.unpack(mask, frame[x : x + step])[0]
            for x in range(0, len(frame), step)
        ]
        return sum(x) % modulus

    return add


ADD11 = adder(2 ** 8)
ADD12 = adder(2 ** 16)
ADD14 = adder(2 ** 32)
ADD22 = wordSum(2 ** 16, 2)
ADD24 = wordSum(2 ** 32, 2)
ADD44 = wordSum(2 ** 32, 4)
CRC16 = Crc16(CRC16, 0x0000, 0x0000, True, True)
CRC16_CCITT = Crc16(CRC16_CCITT, 0xFFFF, 0x0000, False, False)


def CRC32(x):
    return zlib.crc32(x) & 0xFFFFFFFF


def userDefined(x):
    """User defined algorithms are not supported yet."""
    raise NotImplementedError("Checksum method 'XCP_USER_DEFINED' not supported yet.")


ALGO = {
    "XCP_ADD_11": ADD11,
    "XCP_ADD_12": ADD12,
    "XCP_ADD_14": ADD14,
    "XCP_ADD_22": ADD22,
    "XCP_ADD_24": ADD24,
    "XCP_ADD_44": ADD44,
    "XCP_CRC_16": CRC16,
    "XCP_CRC_16_CITT": CRC16_CCITT,
    "XCP_CRC_32": CRC32,
    "XCP_USER_DEFINED": userDefined,
}


def check(frame, algo):
    """Calculate checksum using given algorithm

    Parameters
    ----------
    frame : list of integers
    algo : `ALGO`

    Returns
    -------
    int
    """
    fun = ALGO.get(algo)
    if fun:
        return fun(frame)
    else:
        raise NotImplementedError("Invalid algorithm '{}'.".format(algo))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Use this as a copy-and-paste template for your own scripts.
"""

__copyright__ = """
    pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2009-2020 by Christoph Schueler <cpu12.gems@googlemail.com>

   All Rights Reserved

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""


from pprint import pprint

from pyxcp.cmdline import ArgumentParser


def callout(master, args):
    if args.sk_dll:
        master.seedNKeyDLL = args.sk_dll


ap = ArgumentParser(description="pyXCP skeleton.", callout=callout)

# Add command-line option for seed-and-key DLL.
ap.parser.add_argument(
    "-s",
    "--sk-dll",
    dest="sk_dll",
    help="Seed-and-Key .DLL name",
    type=str,
    default=None,
)

with ap.run() as x:
    x.connect()
    if x.slaveProperties.optionalCommMode:
        # Collect additional properties.
        x.getCommModeInfo()

    # getId() is not strictly required.
    gid = x.getId(0x1)
    slave_name = x.fetch(gid.length)

    # Unlock resources, if necessary.
    # Could be more specific, like cond_unlock("DAQ")
    # Note: Unlocking requires a seed-and-key DLL.
    x.cond_unlock()

    ##
    # Your own code goes here.
    ##

    x.disconnect()

# Print some useful information.
# print("\nSlave properties:")
# print("=================")
# print("ID: '{}'".format(slave_name.decode("utf8")))
# pprint(x.slaveProperties)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
python-can driver for Kvaser interfaces.
"""

__copyright__ = """
    pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2009-2020 by Christoph Schueler <cpu12.gems@googlemail.com>

   All Rights Reserved

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import pyxcp.transport.can as can

import pyxcp.transport.candriver.python_can as python_can


class Kvaser(python_can.PythonCAN, can.CanInterfaceBase):
    """"""

    PARAMETER_MAP = {
        #                        Type    Req'd   Default
        "ACCEPT_VIRTUAL": (bool, False, True),
        "DRIVER_MODE": (bool, False, True),
        "NO_SAMP": (int, False, 1),
        "SJW": (int, False, 2),
        "TSEG1": (int, False, 5),
        "TSEG2": (int, False, 2),
        "SINGLE_HANDLE": (bool, False, True),
        "FD": (bool, False, False),
        "DATA_BITRATE": (int, False, None),
    }

    PARAMETER_TO_KW_ARG_MAP = {
        "ACCEPT_VIRTUAL": "accept_virtual",
        "TSEG1": "tseg1",
        "TSEG2": "tseg2",
        "SJW": "sjw",
        "NO_SAMP": "no_samp",
        "DRIVER_MODE": "driver_mode",
        "SINGLE_HANDLE": "single_handle",
        "FD": "fd",
        "DATA_BITRATE": "data_bitrate",
    }

    def __init__(self):
        super(Kvaser, self).__init__(bustype="kvaser")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Raw XCP traffic recorder.

Data is stored in LZ4 compressed containers.

Examples
--------

See

- ``_ for recording / writing

-  ``_ for reading.
"""

__copyright__ = """
   pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2020-2021 by Christoph Schueler <cpu12.gems.googlemail.com>

   All Rights Reserved

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   s. FLOSS-EXCEPTION.txt
"""


from collections import namedtuple
import enum
import mmap
import os
import pathlib
import struct

import lz4.block as lz4block


FILE_EXTENSION = ".xmraw"  # XCP Measurement / raw data.

MAGIC = b"ASAMINT::XCP_RAW"

FILE_HEADER_STRUCT = struct.Struct("<{:d}sHHHLLLL".format(len(MAGIC)))
FileHeader = namedtuple(
    "FileHeader",
    "magic hdr_size version options num_containers record_count size_compressed size_uncompressed",
)

CONTAINER_HEADER_STRUCT = struct.Struct("<LLL")
ContainerHeader = namedtuple(
    "ContainerHeader", "record_count size_compressed size_uncompressed"
)

DAQ_RECORD_STRUCT = struct.Struct("<BHdL")
DAQRecord = namedtuple("DAQRecord", "category counter timestamp payload")


class XcpLogCategory(enum.IntEnum):
    """ """

    DAQ = 1


class XcpLogFileParseError(Exception):
    """Log file is damaged is some way."""

    pass


class XcpLogFileCapacityExceededError(Exception):
    pass


class XcpLogFileWriter:
    """
    Parameters
    ----------
    file_name: str
        Don't specify extension.

    prealloc: int
        Pre-allocate a sparse file (size in MB).

    chunk_size: int
        Number of kilobytes to collect before compressing.

    compression_level: int
        s. LZ4 documentation.
    """

    def __init__(
        self,
        file_name: str,
        prealloc: int = 10,
        chunk_size: int = 1024,
        compression_level: int = 9,
    ):
        self._is_closed = True
        try:
            self._of = open("{}{}".format(file_name, FILE_EXTENSION), "w+b")
        except Exception as e:
            raise
        else:
            self._of.truncate(1024 * 1024 * prealloc)  # Create sparse file (hopefully).
            self._mapping = mmap.mmap(self._of.fileno(), 0)
        self.container_header_offset = FILE_HEADER_STRUCT.size
        self.current_offset = (
            self.container_header_offset + CONTAINER_HEADER_STRUCT.size
        )
        self.total_size_uncompressed = self.total_size_compressed = 0
        self.container_size_uncompressed = self.container_size_compressed = 0
        self.total_record_count = 0
        self.chunk_size = chunk_size * 1024
        self.num_containers = 0
        self.intermediate_storage = []
        self.compression_level = compression_level
        self.prealloc = prealloc
        self._is_closed = False

    def add_xcp_frames(self, xcp_frames: list):
        for counter, timestamp, raw_data in xcp_frames:
            length = len(raw_data)
            item = DAQ_RECORD_STRUCT.pack(1, counter, timestamp, length) + raw_data
            self.intermediate_storage.append(item)
            self.container_size_uncompressed += len(item)
            if self.container_size_uncompressed > self.chunk_size:
                self._compress_framez()

    def _compress_framez(self):
        compressed_data = lz4block.compress(
            b"".join(self.intermediate_storage), compression=self.compression_level
        )
        record_count = len(self.intermediate_storage)
        hdr = CONTAINER_HEADER_STRUCT.pack(
            record_count, len(compressed_data), self.container_size_uncompressed
        )
        self.set(self.current_offset, compressed_data)
        self.set(self.container_header_offset, hdr)
        self.container_header_offset = self.current_offset + len(compressed_data)
        self.current_offset = (
            self.container_header_offset + CONTAINER_HEADER_STRUCT.size
        )
        self.intermediate_storage = []
        self.total_record_count += record_count
        self.num_containers += 1
        self.total_size_uncompressed += self.container_size_uncompressed
        self.total_size_compressed += len(compressed_data)
        self.container_size_uncompressed = 0
        self.container_size_compressed = 0

    def __del__(self):
        if not self._is_closed:
            self.close()

    def close(self):
        if not self._is_closed:
            if hasattr(self, "_mapping"):
                if self.intermediate_storage:
                    self._compress_framez()
                self._write_header(
                    version=0x0100,
                    options=0x0000,
                    num_containers=self.num_containers,
                    record_count=self.total_record_count,
                    size_compressed=self.total_size_compressed,
                    size_uncompressed=self.total_size_uncompressed,
                )
                self._mapping.flush()
                self._mapping.close()
                self._of.truncate(self.current_offset)
            self._of.close()
            self._is_closed = True

    def set(self, address: int, data: bytes):
        """Write to memory mapped file.

        Parameters
        ----------
        address: int

        data: bytes-like
        """
        length = len(data)
        try:
            self._mapping[address : address + length] = data
        except IndexError as e:
            raise XcpLogFileCapacityExceededError(
                "Maximum file size of {} MBytes exceeded.".format(self.prealloc)
            )

    def _write_header(
        self,
        version,
        options,
        num_containers,
        record_count,
        size_compressed,
        size_uncompressed,
    ):
        hdr = FILE_HEADER_STRUCT.pack(
            MAGIC,
            FILE_HEADER_STRUCT.size,
            version,
            options,
            num_containers,
            record_count,
            size_compressed,
            size_uncompressed,
        )
        self.set(0x00000000, hdr)

    @property
    def compression_ratio(self):
        if self.total_size_compressed:
            return self.total_size_uncompressed / self.total_size_compressed


class XcpLogFileReader:
    """
    Parameters
    ----------
    file_name: str
        Don't specify extension.
    """

    def __init__(self, file_name):
        self._is_closed = True
        try:
            self._log_file = open("{}{}".format(file_name, FILE_EXTENSION), "r+b")
        except Exception as e:
            raise
        else:
            self._mapping = mmap.mmap(self._log_file.fileno(), 0)
        self._is_closed = False
        (
            magic,
            _,
            _,
            _,
            self.num_containers,
            self.total_record_count,
            self.total_size_compressed,
            self.total_size_uncompressed,
        ) = FILE_HEADER_STRUCT.unpack(self.get(0, FILE_HEADER_STRUCT.size))
        if magic != MAGIC:
            raise XcpLogFileParseError("Invalid file magic: '{}'.".format(magic))

    def __del__(self):
        if not self._is_closed:
            self.close()

    @property
    def frames(self):
        """Iterate over all frames in file.

        Yields
        ------
        DAQRecord
        """
        offset = FILE_HEADER_STRUCT.size
        for _ in range(self.num_containers):
            (
                record_count,
                size_compressed,
                size_uncompressed,
            ) = CONTAINER_HEADER_STRUCT.unpack(
                self.get(offset, CONTAINER_HEADER_STRUCT.size)
            )
            offset += CONTAINER_HEADER_STRUCT.size
            uncompressed_data = memoryview(
                lz4block.decompress(self.get(offset, size_compressed))
            )
            frame_offset = 0
            for _ in range(record_count):
                category, counter, timestamp, frame_length = DAQ_RECORD_STRUCT.unpack(
                    uncompressed_data[
                        frame_offset : frame_offset + DAQ_RECORD_STRUCT.size
                    ]
                )
                frame_offset += DAQ_RECORD_STRUCT.size
                frame_data = uncompressed_data[
                    frame_offset : frame_offset + frame_length
                ]  # .tobytes()
                frame_offset += len(frame_data)
                frame = DAQRecord(category, counter, timestamp, frame_data)
                yield frame
            offset += size_compressed

    def get(self, address: int, length: int):
        """Read from memory mapped file.

        Parameters
        ----------
        address: int

        length: int

        Returns
        -------
        memoryview
        """
        return self._mapping[address : address + length]

    def close(self):
        if hasattr(self, "self._mapping"):
            self._mapping.close()
        self._log_file.close()
        self._is_closed = True

    @property
    def compression_ratio(self):
        if self.total_size_compressed:
            return self.total_size_uncompressed / self.total_size_compressed



import pytest

import pyxcp.transport.base as tr


def test_factory_works():
    assert isinstance(tr.createTransport("eth"), tr.BaseTransport)
    assert isinstance(tr.createTransport("sxi"), tr.BaseTransport)
    assert isinstance(
        tr.createTransport(
            "can",
            config={
                "CAN_ID_MASTER": 1,
                "CAN_ID_SLAVE": 2,
                "CAN_DRIVER": "MockCanInterface",
            },
        ),
        tr.BaseTransport,
    )


def test_factory_works_case_insensitive():
    assert isinstance(tr.createTransport("ETH"), tr.BaseTransport)
    assert isinstance(tr.createTransport("SXI"), tr.BaseTransport)
    assert isinstance(
        tr.createTransport(
            "CAN",
            config={
                "CAN_ID_MASTER": 1,
                "CAN_ID_SLAVE": 2,
                "CAN_DRIVER": "MockCanInterface",
            },
        ),
        tr.BaseTransport,
    )


def test_factory_invalid_transport_name_raises():
    with pytest.raises(ValueError):
        tr.createTransport("xCp")


def test_transport_names():
    transports = tr.availableTransports()

    assert "can" in transports
    assert "eth" in transports
    assert "sxi" in transports


def test_transport_names_are_lower_case_only():
    transports = tr.availableTransports()

    assert "CAN" not in transports
    assert "ETH" not in transports
    assert "SXI" not in transports


def test_transport_classes():
    transports = tr.availableTransports()

    assert issubclass(transports.get("can"), tr.BaseTransport)
    assert issubclass(transports.get("eth"), tr.BaseTransport)
    assert issubclass(transports.get("sxi"), tr.BaseTransport)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from collections import OrderedDict
from pprint import pprint

__copyright__ = """
    pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2009-2019 by Christoph Schueler <cpu12.gems@googlemail.com>

   All Rights Reserved

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import array
from collections import namedtuple
import time

from pyxcp import checksum
from pyxcp import types
from pyxcp import transport
from pyxcp.dllif import getKey
from pyxcp.master import Master
from pyxcp.utils import hexDump


def test():
    xm = Master(transport.Eth("localhost", connected=False))
    xm.connect()

    print("calpag ?", xm.supportsCalpag)
    print("daq ?", xm.supportsDaq)
    print("pgm ?", xm.supportsPgm)
    print("stim ?", xm.supportsStim)

    xm.getStatus()
    xm.synch()
    xm.getCommModeInfo()

    result = xm.getID(0x04)
    print(result)
    # xm.upload(result.length)

    #    unlock(xm, 4)
    # length, seed = xm.getSeed(0x7ba, 0, 4)
    # print("SEED: ", hexDump(seed), flush = True)
    # _, kee = skloader.getKey(b"SeedNKeyXcp.dll", 4, seed)
    # print("KEE:", kee)
    # print(xm.unlock(0x7a, len(kee), kee))

    print("DAQ_PROC_INFO: ", xm.getDaqProcessorInfo())

    print("TIMESTAMP: {:04X}".format(xm.getDaqClock()))

    # print("readDAQ:", xm.readDaq())

    print("daqResolutionInfo", xm.getDaqResolutionInfo())

    dpi = xm.getDaqProcessorInfo()

    for ecn in range(dpi.maxEventChannel):
        eci = xm.getEventChannelInfo(ecn)
        print(eci)
        data = xm.upload(eci.eventChannelNameLength)
        print("EventChannelName:", data.decode("latin1"))

    # print("GetDAQListInfo", xm.getDaqListInfo(0))
    xm.freeDaq()
    print("AllocDAQ:", xm.allocDaq(2))

    xm.setMta(0x1C0000)
    print("CS:", xm.buildChecksum(4742))

    #    unlock(xm, 1)
    #    verify(xm, 0x1C0000, 128)

    # print("PS:", xm.programStart()) # ERR_ACCESS_LOCKED

    xm.disconnect()
    xm.close()


def timecode(ticks, mode):
    units = {
        "DAQ_TIMESTAMP_UNIT_1PS": -12,
        "DAQ_TIMESTAMP_UNIT_10PS": -11,
        "DAQ_TIMESTAMP_UNIT_100PS": -10,
        "DAQ_TIMESTAMP_UNIT_1NS": -9,
        "DAQ_TIMESTAMP_UNIT_10NS": -8,
        "DAQ_TIMESTAMP_UNIT_100NS": -7,
        "DAQ_TIMESTAMP_UNIT_1US": -6,
        "DAQ_TIMESTAMP_UNIT_10US": -5,
        "DAQ_TIMESTAMP_UNIT_100US": -4,
        "DAQ_TIMESTAMP_UNIT_1MS": -3,
        "DAQ_TIMESTAMP_UNIT_10MS": -2,
        "DAQ_TIMESTAMP_UNIT_100MS": -1,
        "DAQ_TIMESTAMP_UNIT_1S": 0,
    }
    return (10 ** units[mode.timestampMode.unit]) * mode.timestampTicks * ticks


def cstest():
    tr = transport.Eth(
        "localhost", port=5555, protocol="UDP", loglevel="WARN"
    )  # "DEBUG"
    # tr = transport.SxI("COM27", 115200, loglevel = "WARN")
    with Master(tr) as xm:
        #    tm = Timing()

        conn = xm.connect()
        print(conn, flush=True)

        print("calpag ?", xm.supportsCalpag)
        print("daq ?", xm.supportsDaq)
        print("pgm ?", xm.supportsPgm)
        print("stim ?", xm.supportsStim)

        print(xm.getStatus(), flush=True)
        xm.synch()

        if conn.commModeBasic.optional:
            print(xm.getCommModeInfo(), flush=True)
        else:
            print("No details on connection.")

        gid = xm.getID(0x1)
        # gid = xm.getID(0xdb)
        print("GET_ID:", gid, flush=True)
        result = xm.fetch(gid.length)
        print(result.decode("utf8"))

        #        bench(xm)

        # gid = xm.getID(0xdb)
        # print("DB", gid)
        #        result = xm.upload()
        #        print("ID: '{}'".format(result.decode("utf8")))

        #        start = time.perf_counter()
        #        result = xm.fetch(gid.length, 252)
        #        stop = time.perf_counter()
        #        print("ETA: {:.2f} s - rate: {:.2f} kB/s".format(stop - start, (gid.length / (stop - start)) / 1000 ) )

        #        print("ID: '{}'".format(result.decode("utf8")))

        xm.setMta(0x0040C280)
        bhv = xm.fetch(0x100, 16)
        print(bhv)

        resInfo = xm.getDaqResolutionInfo()
        print(resInfo, flush=True)
        xm.getDaqProcessorInfo()

        #    print("CS:", xm.buildChecksum(4711))

        start = xm.getDaqClock()
        print("Timestamp / Start: {}".format(start))
        """
        length, seed = xm.getSeed(0, 0xff)
        print("SEED: ", hexDump(seed))
        resultCode, kee = getKey("SeedNKeyXcp.dll", 1, seed)
        if resultCode == 0:
            res = xm.unlock(len(kee), kee)
            print(res)
        """

        startMeasurement(xm)
        ##
        # xm.freeDaq()
        ##    print("AllocDAQ:", xm.allocDaq(2))
        ##
        ##    print("allocOdt", xm.allocOdt(1, 5))
        ##    print("allocOdt", xm.allocOdt(0, 4))
        ##    print("allocOdt", xm.allocOdt(1, 3))
        ##
        ##    print("allocOdt", xm.allocOdtEntry(1, 3, 5))
        ##    print("allocOdt", xm.allocOdtEntry(0, 1, 2))
        ##    print("allocOdt", xm.allocOdtEntry(0, 3, 6))
        ##    print("allocOdt", xm.allocOdtEntry(1, 1, 5))
        ##

        # xm.freeDaq()

        #    for _ in range(10):
        #        tm.start()
        #        time.sleep(0.250)
        #        tm.stop()
        #        stop = xm.getDaqClock()
        #        print("trueValue: {}".format(timecode(stop - start, resInfo)))
        #        print("Timestamp / Diff: {}".format(stop - start))

        print("Timer")
        print("=====")
        #    print(tm)

        print(xm.setMta(0x1C0000), flush=True)
        # print(xm.setRequest(0x3, 1))
        print(xm.disconnect(), flush=True)
        # xm.close()
        print("XCP roundtrip timing")
        print("=" * 20)
    for _ in range(tr.daqQueue.qsize()):
        daq = tr.daqQueue.get(timeout=1.0)
        print(types.DAQ.parse(daq))

    # dq = construct.Struct(
    #     "odt" / construct.Byte,
    #     "dl" / construct.Byte,
    #     "data" / construct.GreedyBytes,
    # )

    # ev = tr.evQueue.get(timeout = 1.0)
    # print(ev)


#   print(tr.timing)


# import pandas as pd


def bench(xm):
    import pandas as pd

    result = OrderedDict()
    for pn in range(8, 257, 8):
        # for pn in range(8, 257, 32):
        result[pn] = []
        for i in range(10):
            gid = xm.getID(0x4)
            start = time.perf_counter()
            xm.fetch(gid.length, pn)
            stop = time.perf_counter()
            eta = stop - start
            result[pn].append(eta)
    pprint(result)
    df = pd.DataFrame(result)
    df.to_csv("payloads01.csv")


# A2L linearizer


DaqEntry = namedtuple("DaqEntry", "daq odt entry bitoff size ext addr")


def startMeasurement(cl):
    print(cl.getDaqProcessorInfo())
    cl.freeDaq()
    cl.allocDaq(2)

    # cl.allocOdt(1, 13)
    # cl.allocOdt(0, 2)

    cl.allocOdt(0, 13)
    cl.allocOdt(1, 2)

    cl.allocOdtEntry(0, 0, 1)
    cl.allocOdtEntry(0, 1, 1)
    cl.allocOdtEntry(0, 2, 1)
    cl.allocOdtEntry(0, 3, 1)
    cl.allocOdtEntry(0, 4, 1)
    cl.allocOdtEntry(0, 5, 1)
    cl.allocOdtEntry(0, 6, 1)
    cl.allocOdtEntry(0, 7, 1)
    cl.allocOdtEntry(0, 8, 1)
    cl.allocOdtEntry(0, 9, 1)
    cl.allocOdtEntry(0, 10, 1)
    cl.allocOdtEntry(0, 11, 3)
    cl.allocOdtEntry(0, 12, 5)

    cl.allocOdtEntry(1, 0, 1)
    cl.allocOdtEntry(1, 1, 1)

    de0 = (
        DaqEntry(daq=0, odt=0, entry=0, bitoff=255, size=2, ext=0, addr=0x001BE068),
        DaqEntry(daq=0, odt=1, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE06A),
        DaqEntry(daq=0, odt=2, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE070),
        DaqEntry(daq=0, odt=3, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE076),
        DaqEntry(daq=0, odt=4, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE07C),
        DaqEntry(daq=0, odt=5, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE082),
        DaqEntry(daq=0, odt=6, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE088),
        DaqEntry(daq=0, odt=7, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE08E),
        DaqEntry(daq=0, odt=8, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE094),
        DaqEntry(daq=0, odt=9, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE09A),
        DaqEntry(daq=0, odt=10, entry=0, bitoff=255, size=6, ext=0, addr=0x001BE0A0),
        DaqEntry(daq=0, odt=11, entry=0, bitoff=255, size=2, ext=0, addr=0x001BE0A6),
        DaqEntry(daq=0, odt=11, entry=1, bitoff=255, size=1, ext=0, addr=0x001BE0CF),
        DaqEntry(daq=0, odt=11, entry=2, bitoff=255, size=3, ext=0, addr=0x001BE234),
        DaqEntry(daq=0, odt=12, entry=0, bitoff=255, size=1, ext=0, addr=0x001BE237),
        DaqEntry(daq=0, odt=12, entry=1, bitoff=255, size=1, ext=0, addr=0x001BE24F),
        DaqEntry(daq=0, odt=12, entry=2, bitoff=255, size=1, ext=0, addr=0x001BE269),
        DaqEntry(daq=0, odt=12, entry=3, bitoff=255, size=1, ext=0, addr=0x001BE5A3),
        DaqEntry(daq=0, odt=12, entry=4, bitoff=255, size=1, ext=0, addr=0x001C0003),
        DaqEntry(daq=1, odt=0, entry=0, bitoff=255, size=2, ext=0, addr=0x001C002C),
        DaqEntry(daq=1, odt=1, entry=0, bitoff=255, size=2, ext=0, addr=0x001C002E),
    )
    """
-> CONNECT mode=0
<- 0xFF version=01h/01h, maxcro=FFh, maxdto=5DCh, resource=1D, mode=C0
-> GET_STATUS
<- 0xFF sessionstatus=00h, protectionstatus=1D
-> SYNC
<- 0xFE 0x00
-> GET_COMM_MODE_INFO
<- 0xFF
-> GET_ID type=1
<- 0xFF mode=0,len=6
-> UPLOAD size=6
<- 0xFF data=58 43 50 73 69 6D
-> GET_DAQ_RESOLUTION_INFO
<- 0xFF , mode=44h, , ticks=0Ah
-> GET_DAQ_CLOCK
<- 0xFF time=36779B9Ah
-> GET_SEED resource=FFh
<- 0xFF length=0Ah, seed=7C4FBDFBA3D8
-> UNLOCK key=3C7012190000
<- 0xFF
-> GET_DAQ_PROCESSOR_INFO
<- 0xFF

-> FREE_DAQ
<- 0xFF
-> ALLOC_DAQ count=2
[XcpAllocMemory] 17/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT daq=0, count=13
[XcpAllocMemory] 121/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT daq=1, count=2
[XcpAllocMemory] 137/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=0, count=1
[XcpAllocMemory] 142/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=1, count=1
[XcpAllocMemory] 147/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=2, count=1
[XcpAllocMemory] 152/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=3, count=1
[XcpAllocMemory] 157/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=4, count=1
[XcpAllocMemory] 162/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=5, count=1
[XcpAllocMemory] 167/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=6, count=1
[XcpAllocMemory] 172/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=7, count=1
[XcpAllocMemory] 177/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=8, count=1
[XcpAllocMemory] 182/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=9, count=1
[XcpAllocMemory] 187/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=10, count=1
[XcpAllocMemory] 192/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=11, count=3
[XcpAllocMemory] 207/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=0, odt=12, count=5
[XcpAllocMemory] 232/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=1, odt=0, count=1
[XcpAllocMemory] 237/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> ALLOC_ODT_ENTRY daq=1, odt=1, count=1
[XcpAllocMemory] 242/65535 Bytes used
[XcpAllocMemory] Queuesize=127
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=0,idx=0
<- 0xFF
-> WRITE_DAQ size=2,addr=001BE068h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=1,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE06Ah,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=2,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE070h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=3,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE076h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=4,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE07Ch,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=5,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE082h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=6,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE088h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=7,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE08Eh,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=8,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE094h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=9,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE09Ah,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=10,idx=0
<- 0xFF
-> WRITE_DAQ size=6,addr=001BE0A0h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=11,idx=0
<- 0xFF
-> WRITE_DAQ size=2,addr=001BE0A6h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=11,idx=1
<- 0xFF
-> WRITE_DAQ size=1,addr=001BE0CFh,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=11,idx=2
<- 0xFF
-> WRITE_DAQ size=3,addr=001BE234h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=12,idx=0
<- 0xFF
-> WRITE_DAQ size=1,addr=001BE237h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=12,idx=1
<- 0xFF
-> WRITE_DAQ size=1,addr=001BE24Fh,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=12,idx=2
<- 0xFF
-> WRITE_DAQ size=1,addr=001BE269h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=12,idx=3
<- 0xFF
-> WRITE_DAQ size=1,addr=001BE5A3h,00h
<- 0xFF
-> SET_DAQ_PTR daq=0,odt=12,idx=4
<- 0xFF
-> WRITE_DAQ size=1,addr=001C0003h,00h
<- 0xFF

-> SET_DAQ_PTR daq=1,odt=0,idx=0
<- 0xFF
-> WRITE_DAQ size=2,addr=001C002Ch,00h
<- 0xFF
-> SET_DAQ_PTR daq=1,odt=1,idx=0
<- 0xFF
-> WRITE_DAQ size=2,addr=001C002Eh,00h
<- 0xFF
-> SET_DAQ_LIST_MODE daq=0, mode=10h, prescaler=1, eventchannel=1, priority=1
<- 0xFF
DAQ 0:
 eventchannel=0001h, prescaler=1, firstOdt=0, lastOdt=12, flags=11h
 firstPid=00h
  ODT 0 (0):
   pid=0:
   firstOdtEntry=0,lastOdtEntry=0:
   [005BE068h,2]
  ODT 1 (1):
   pid=1:
   firstOdtEntry=1,lastOdtEntry=1:
   [005BE06Ah,6]
  ODT 2 (2):
   pid=2:
   firstOdtEntry=2,lastOdtEntry=2:
   [005BE070h,6]
  ODT 3 (3):
   pid=3:
   firstOdtEntry=3,lastOdtEntry=3:
   [005BE076h,6]
  ODT 4 (4):
   pid=4:
   firstOdtEntry=4,lastOdtEntry=4:
   [005BE07Ch,6]
  ODT 5 (5):
   pid=5:
   firstOdtEntry=5,lastOdtEntry=5:
   [005BE082h,6]
  ODT 6 (6):
   pid=6:
   firstOdtEntry=6,lastOdtEntry=6:
   [005BE088h,6]
  ODT 7 (7):
   pid=7:
   firstOdtEntry=7,lastOdtEntry=7:
   [005BE08Eh,6]
  ODT 8 (8):
   pid=8:
   firstOdtEntry=8,lastOdtEntry=8:
   [005BE094h,6]
  ODT 9 (9):
   pid=9:
   firstOdtEntry=9,lastOdtEntry=9:
   [005BE09Ah,6]
  ODT 10 (10):
   pid=10:
   firstOdtEntry=10,lastOdtEntry=10:
   [005BE0A0h,6]
  ODT 11 (11):
   pid=11:
   firstOdtEntry=11,lastOdtEntry=13:
   [005BE0A6h,2]
   [005BE0CFh,1]
   [005BE234h,3]
  ODT 12 (12):
   pid=12:
   firstOdtEntry=14,lastOdtEntry=18:
   [005BE237h,1]
   [005BE24Fh,1]
   [005BE269h,1]
   [005BE5A3h,1]
   [005C0003h,1]
Queue:
 xcp.pQueue        = 59DF1A
 xcp.QueueSize     = 127
 xcp.QueueLen      = 0
 xcp.QueueRp       = 0
-> START_STOP mode=02h, daq=0
<- 0xFF
-> SET_DAQ_LIST_MODE daq=1, mode=10h, prescaler=1, eventchannel=2, priority=2
<- 0xFF
DAQ 1:
 eventchannel=0002h, prescaler=1, firstOdt=13, lastOdt=14, flags=11h
 firstPid=0Dh
  ODT 0 (13):
   pid=13:
   firstOdtEntry=19,lastOdtEntry=19:
   [005C002Ch,2]
  ODT 1 (14):
   pid=14:
   firstOdtEntry=20,lastOdtEntry=20:
   [005C002Eh,2]
Queue:
 xcp.pQueue        = 59DF1A
 xcp.QueueSize     = 127
 xcp.QueueLen      = 0
 xcp.QueueRp       = 0
-> START_STOP mode=02h, daq=1
<- 0xFF
-> CC_START_STOP_SYNCH mode=01h
<- 0xFF
-> CC_START_STOP_SYNCH mode=00h
<- 0xFF
-> SET_MTA addr=001C0000h, addrext=00h
<- 0xFF
-> DISCONNECT
<- 0xFF
    """

    for daq, odt, entry, bitoff, size, ext, addr in de0:
        cl.setDaqPtr(daq, odt, entry)
        cl.writeDaq(bitoff, size, ext, addr)
    """
    SET_DAQ_LIST_MODE mode=10h daq=0 event=1 prescaler=1 priority=1
    START_STOP_DAQ_LIST mode=02h daq=0
    SET_DAQ_LIST_MODE mode=10h daq=1 event=2 prescaler=1 priority=2
    START_STOP_DAQ_LIST mode=02h daq=1
    GET_DAQ_CLOCK
    GET_DAQ_CLOCK
    START_STOP_SYNCH mode=01h
    """
    cl.setDaqListMode(0x10, 0, 1, 1, 0)  # , 1)
    print("startStopDaqList #0", cl.startStopDaqList(0x02, 0))
    cl.setDaqListMode(0x10, 1, 2, 1, 0)  # , 2)
    print("startStopDaqList #1", cl.startStopDaqList(0x02, 1))
    cl.startStopSynch(0x01)

    time.sleep(3.0)

    cl.startStopSynch(0x00)


if __name__ == "__main__":
    cstest()

"""
FREE_DAQ
ALLOC_DAQ count=2
ALLOC_ODT daq=0 count=13
ALLOC_ODT daq=1 count=2

ALLOC_ODT_ENTRY daq=0 odt=0 count=1
ALLOC_ODT_ENTRY daq=0 odt=1 count=1
ALLOC_ODT_ENTRY daq=0 odt=2 count=1
ALLOC_ODT_ENTRY daq=0 odt=3 count=1
ALLOC_ODT_ENTRY daq=0 odt=4 count=1
ALLOC_ODT_ENTRY daq=0 odt=5 count=1
ALLOC_ODT_ENTRY daq=0 odt=6 count=1
ALLOC_ODT_ENTRY daq=0 odt=7 count=1
ALLOC_ODT_ENTRY daq=0 odt=8 count=1
ALLOC_ODT_ENTRY daq=0 odt=9 count=1
ALLOC_ODT_ENTRY daq=0 odt=10 count=1
ALLOC_ODT_ENTRY daq=0 odt=11 count=3
ALLOC_ODT_ENTRY daq=0 odt=12 count=5

ALLOC_ODT_ENTRY daq=1 odt=0 count=1
ALLOC_ODT_ENTRY daq=1 odt=1 count=1

SET_DAQ_PTR daq=0 odt=0 entry=0
WRITE_DAQ bitoff=255 size=2 ext=0 addr=001BE068h
SET_DAQ_PTR daq=0 odt=1 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE06Ah
SET_DAQ_PTR daq=0 odt=2 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE070h
SET_DAQ_PTR daq=0 odt=3 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE076h
SET_DAQ_PTR daq=0 odt=4 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE07Ch
SET_DAQ_PTR daq=0 odt=5 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE082h
SET_DAQ_PTR daq=0 odt=6 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE088h
SET_DAQ_PTR daq=0 odt=7 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE08Eh
SET_DAQ_PTR daq=0 odt=8 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE094h
SET_DAQ_PTR daq=0 odt=9 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE09Ah
SET_DAQ_PTR daq=0 odt=10 entry=0
WRITE_DAQ bitoff=255 size=6 ext=0 addr=001BE0A0h
SET_DAQ_PTR daq=0 odt=11 entry=0
WRITE_DAQ bitoff=255 size=2 ext=0 addr=001BE0A6h
WRITE_DAQ bitoff=255 size=1 ext=0 addr=001BE0CFh
WRITE_DAQ bitoff=255 size=3 ext=0 addr=001BE234h
SET_DAQ_PTR daq=0 odt=12 entry=0
WRITE_DAQ bitoff=255 size=1 ext=0 addr=001BE237h
WRITE_DAQ bitoff=255 size=1 ext=0 addr=001BE24Fh
WRITE_DAQ bitoff=255 size=1 ext=0 addr=001BE269h
WRITE_DAQ bitoff=255 size=1 ext=0 addr=001BE5A3h
WRITE_DAQ bitoff=255 size=1 ext=0 addr=001C0003h
SET_DAQ_PTR daq=1 odt=0 entry=0
WRITE_DAQ bitoff=255 size=2 ext=0 addr=001C002Ch
SET_DAQ_PTR daq=1 odt=1 entry=0
WRITE_DAQ bitoff=255 size=2 ext=0 addr=001C002Eh

SET_DAQ_LIST_MODE mode=10h daq=0 event=1 prescaler=1 priority=1
START_STOP_DAQ_LIST mode=02h daq=0
SET_DAQ_LIST_MODE mode=10h daq=1 event=2 prescaler=1 priority=2
START_STOP_DAQ_LIST mode=02h daq=1
GET_DAQ_CLOCK
GET_DAQ_CLOCK
START_STOP_SYNCH mode=01h
"""
